@@ -16,7 +16,7 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
  */
 
 
@@ -25,8 +25,10 @@
 
 #include <GNUstepGUI/GSFontInfo.h>
 #include <cairo.h>
+#include "cairo/CairoFaceInfo.h"
 
 @interface CairoFontEnumerator : GSFontEnumerator
++ (CairoFaceInfo *) fontWithName: (NSString *)name;
 @end
 
 #endif
