@@ -21,7 +21,8 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #ifndef _GSGState_h_INCLUDE
@@ -47,7 +48,7 @@ typedef enum {
   COLOR_BOTH = 3               /* COLOR_BOTH = COLOR_FILL || COLOR_STROKE */
 } color_state_t;
 
-@interface GSGState : NSObject
+@interface GSGState : NSObject <NSCopying>
 {
 @public
   GSContext *drawcontext;
