@@ -19,7 +19,8 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #ifndef	INCLUDED_XGGENERIC_H
@@ -50,6 +51,22 @@ typedef struct {
   Atom win_topmenu_atom;
 } XGWMWinTypes;
 
+typedef struct {
+  Atom net_wm_state_atom;
+  Atom net_wm_state_skip_taskbar_atom;
+} XGWMNetStates;
+
+/*
+ * Frame offsets for window inside parent decoration window.
+ */
+typedef struct {
+  short	l;	// offset from left
+  short	r;	// offset from right
+  short	t;	// offset from top
+  short	b;	// offset from bottom
+  BOOL	known;	// obtained from Reparent event or just guessed?
+} Offsets;
+
 /*
  * Structure containing ivars that are common to all X backend contexts.
  */
@@ -77,7 +94,6 @@ struct XGGeneric {
   long			currentFocusWindow;
   long			desiredFocusWindow;
   unsigned long		focusRequestNumber;
-  unsigned long         desiredOrderedWindow;
   unsigned char		lMouse;
   unsigned char		mMouse;
   unsigned char		rMouse;
@@ -88,8 +104,9 @@ struct XGGeneric {
   int			rMouseMask;
   Window		appRootWindow;
   void			*cachedWindow;	// last gswindow_device_t used.
-  XPoint                parent_offset;  // WM defined parent info
+  Offsets		offsets[16];
   XGWMWinTypes          wintypes;
+  XGWMNetStates		netstates;
 };
 
 /* GNOME Window layers */
