@@ -39,7 +39,7 @@
 windows are known to be totally opaque, we can optimize in many ways
 (see big table at the end of blit.m). Will set dst_need_alpha and blit_func
 if necessary. Returns new operation, or -1 it it's a noop. */
--(int) _composite_func: (BOOL)src_opaque : (BOOL)src_transparent
+- (int) _composite_func: (BOOL)src_opaque : (BOOL)src_transparent
 	: (BOOL)dst_opaque : (BOOL *)dst_needs_alpha
 	: (int)op : (void (**)(composite_run_t *c, int num))blit_func_r
 {
@@ -217,27 +217,28 @@ if necessary. Returns new operation, or -1 it it's a noop. */
 
 static void copy_oo(composite_run_t *c, int num)
 {
-	memcpy(c->dst,c->src,num*DI.bytes_per_pixel);
+  memcpy(c->dst, c->src, num * DI.bytes_per_pixel);
 }
 
 static void copy_oa(composite_run_t *c, int num)
 {
-	memcpy(c->dst,c->src,num*DI.bytes_per_pixel);
-	if (DI.inline_alpha)
-	{
-		unsigned char *dsta=c->dst+DI.inline_alpha_ofs;
-		for (;num;num--,dsta+=4)
-			*dsta=0xff;
-	}
-	else
-		memset(c->dsta,0xff,num);
+  memcpy(c->dst, c->src, num * DI.bytes_per_pixel);
+  if (DI.inline_alpha)
+    {
+      unsigned char *dsta = c->dst + DI.inline_alpha_ofs;
+
+      for (; num; num--, dsta += 4)
+	*dsta = 0xff;
+    }
+  else
+    memset(c->dsta, 0xff, num);
 }
 
 static void copy_aa(composite_run_t *c, int num)
 {
-	memcpy(c->dst,c->src,num*DI.bytes_per_pixel);
-	if (!DI.inline_alpha)
-		memcpy(c->dsta,c->srca,num);
+  memcpy(c->dst,c->src,num*DI.bytes_per_pixel);
+  if (!DI.inline_alpha)
+    memcpy(c->dsta, c->srca, num);
 }
 
 
@@ -259,7 +260,7 @@ typedef struct
 } rect_trace_t;
 
 static void _rect_setup(rect_trace_t *t, NSRect r, int cx0, int cx1,
-		 NSAffineTransform *ctm, int up, int *y0, int wi_sy)
+		 NSAffineTransform *ctm, int up, int *y0, NSPoint offset)
 {
   float fx[4], fy[4];
   NSPoint p;
@@ -268,16 +269,16 @@ static void _rect_setup(rect_trace_t *t, NSRect r, int cx0, int cx1,
   t->cx1 = cx1;
 
   p = r.origin;
-  p = [ctm pointInMatrixSpace: p];
+  p = [ctm transformPoint: p];
   fx[0] = p.x; fy[0] = p.y;
   p = r.origin; p.x += r.size.width;
-  p = [ctm pointInMatrixSpace: p];
+  p = [ctm transformPoint: p];
   fx[1] = p.x; fy[1] = p.y;
   p = r.origin; p.x += r.size.width; p.y += r.size.height;
-  p = [ctm pointInMatrixSpace: p];
+  p = [ctm transformPoint: p];
   fx[2] = p.x; fy[2] = p.y;
   p = r.origin; p.y += r.size.height;
-  p = [ctm pointInMatrixSpace: p];
+  p = [ctm transformPoint: p];
   fx[3] = p.x; fy[3] = p.y;
 
   if (fabs(fx[0] - floor(fx[0] + .5)) < 0.001) fx[0] = floor(fx[0] + .5);
@@ -289,10 +290,10 @@ static void _rect_setup(rect_trace_t *t, NSRect r, int cx0, int cx1,
   if (fabs(fy[2] - floor(fy[2] + .5)) < 0.001) fy[2] = floor(fy[2] + .5);
   if (fabs(fy[3] - floor(fy[3] + .5)) < 0.001) fy[3] = floor(fy[3] + .5);
 
-  t->x[0] = floor(fx[0]); t->y[0] = wi_sy - floor(fy[0]);
-  t->x[1] = floor(fx[1]); t->y[1] = wi_sy - floor(fy[1]);
-  t->x[2] = floor(fx[2]); t->y[2] = wi_sy - floor(fy[2]);
-  t->x[3] = floor(fx[3]); t->y[3] = wi_sy - floor(fy[3]);
+  t->x[0] = floor(fx[0]) - offset.x; t->y[0] = offset.y - floor(fy[0]);
+  t->x[1] = floor(fx[1]) - offset.x; t->y[1] = offset.y - floor(fy[1]);
+  t->x[2] = floor(fx[2]) - offset.x; t->y[2] = offset.y - floor(fy[2]);
+  t->x[3] = floor(fx[3]) - offset.x; t->y[3] = offset.y - floor(fy[3]);
 
   /* If we're tracing the 'other way', we just flip the y-coordinates
   and unflip when returning them */
@@ -453,7 +454,6 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   if (!wi || !wi->data || !ags->wi || !ags->wi->data) return;
   if (all_clipped) return;
 
-
   {
     BOOL dst_needs_alpha;
     op = [self _composite_func: !ags->wi->has_alpha : NO
@@ -492,12 +492,12 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   cx1 = clip_x1;
   cy1 = clip_y1;
 
-  sp = [ags->ctm pointInMatrixSpace: aRect.origin];
-  sp.x = floor(sp.x);
-  sp.y = floor(ags->wi->sy - sp.y);
-  dp = [ctm pointInMatrixSpace: aPoint];
-  dp.x = floor(dp.x);
-  dp.y = floor(wi->sy - dp.y);
+  sp = [ags->ctm transformPoint: aRect.origin];
+  sp.x = floor(sp.x - ags->offset.x);
+  sp.y = floor(ags->offset.y - sp.y);
+  dp = [ctm transformPoint: aPoint];
+  dp.x = floor(dp.x - offset.x);
+  dp.y = floor(offset.y - dp.y);
 
   if (dp.x - cx0 > sp.x)
     cx0 = dp.x - sp.x;
@@ -613,7 +613,7 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   {
     int ry;
     int x0, x1;
-    _rect_setup(&state, aRect, sx0, sx0 + cx1, ags->ctm, order, &ry, ags->wi->sy);
+    _rect_setup(&state, aRect, sx0, sx0 + cx1, ags->ctm, order, &ry, ags->offset);
     if (order)
       {
 	if (ry < sy0)
@@ -874,12 +874,12 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   cx1 = clip_x1;
   cy1 = clip_y1;
 
-  sp = [ags->ctm pointInMatrixSpace: aRect.origin];
-  sp.x = floor(sp.x);
-  sp.y = floor(ags->wi->sy - sp.y);
-  dp = [ctm pointInMatrixSpace: aPoint];
-  dp.x = floor(dp.x);
-  dp.y = floor(wi->sy - dp.y);
+  sp = [ags->ctm transformPoint: aRect.origin];
+  sp.x = floor(sp.x - ags->offset.x);
+  sp.y = floor(ags->offset.y - sp.y);
+  dp = [ctm transformPoint: aPoint];
+  dp.x = floor(dp.x - offset.x);
+  dp.y = floor(offset.y - dp.y);
 
   if (dp.x - cx0 > sp.x)
     cx0 = dp.x - sp.x;
@@ -987,7 +987,7 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   {
     int ry;
     int x0, x1;
-    _rect_setup(&state, aRect, sx0, sx0 + cx1, ags->ctm, order, &ry, ags->wi->sy);
+    _rect_setup(&state, aRect, sx0, sx0 + cx1, ags->ctm, order, &ry, ags->offset);
     if (order)
       {
 	if (ry < sy0)
@@ -1254,7 +1254,7 @@ static BOOL _rect_advance(rect_trace_t *t, int *x0, int *x1)
   {
     int ry;
     int x0, x1;
-    _rect_setup(&state, aRect, cx0, cx0 + cx1, ctm, 0, &ry, wi->sy);
+    _rect_setup(&state, aRect, cx0, cx0 + cx1, ctm, 0, &ry, offset);
     if (ry >= cy0 + cy1)
       return;
     delta = ry - cy0;
