@@ -16,7 +16,8 @@
 
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02111 USA.
  */
 #include "cairo/CairoContext.h"
 #include "cairo/CairoGState.h"
@@ -30,10 +31,12 @@
 #include "cairo/XGCairoGlitzSurface.h"
 #else
 #include "cairo/XGCairoSurface.h"
+#include "cairo/XGCairoXImageSurface.h"
 #endif
 
 #define CGSTATE ((CairoGState *)gstate)
 
+@class XWindowBuffer;
 
 @implementation CairoContext
 
@@ -45,28 +48,20 @@
 #ifdef USE_GLITZ
   [CairoSurface setDefaultSurfaceClass: [XGCairoGlitzSurface class]];
 #else
-  [CairoSurface setDefaultSurfaceClass: [XGCairoSurface class]];
+//  [CairoSurface setDefaultSurfaceClass: [XGCairoSurface class]];
+  [CairoSurface setDefaultSurfaceClass: [XGCairoXImageSurface class]];
 #endif
   [GSFontEnumerator setDefaultClass: [CairoFontEnumerator class]];
   [GSFontInfo setDefaultClass: [CairoFontInfo class]];
 }
 
-- (void) GSWSetViewIsFlipped: (BOOL)flipped
-{
-  if (gstate)
-    {
-      CGSTATE->_viewIsFlipped = flipped;
-    }
-}
-
 - (id) initWithContextInfo: (NSDictionary *)info
 {
   NSString *contextType;
 
-  [super initWithContextInfo:info];
-
   contextType = [info objectForKey:
 			 NSGraphicsContextRepresentationFormatAttributeName];
+  self = [super initWithContextInfo: info];
   if (contextType)
     {
       /* Most likely this is a PS or PDF context, so just return what
@@ -84,6 +79,26 @@
   XFlush([(XGServer *)server xDisplay]);
 }
 
+/* Private backend methods */
++(void) handleExposeRect: (NSRect)rect forDriver: (void *)driver
+{
+  [(XWindowBuffer *)driver _exposeRect: rect];
+}
+
+#ifdef XSHM
+
++(void) _gotShmCompletion: (Drawable)d
+{
+  [XWindowBuffer _gotShmCompletion: d];
+}
+
+-(void) gotShmCompletion: (Drawable)d
+{
+  [XWindowBuffer _gotShmCompletion: d];
+}
+
+#endif
+
 @end 
 
 @implementation CairoContext (Ops) 
