@@ -478,29 +478,29 @@ fpfloat(FILE *stream, float f)
 
 - (void) GSSetCTM: (NSAffineTransform *)ctm
 {
-  float m[6];
-  [ctm getMatrix: m];
+  NSAffineTransformStruct matrix = [ctm transformStruct];
+
   fprintf(gstream, "[");
-  fpfloat(gstream, m[0]);
-  fpfloat(gstream, m[1]);
-  fpfloat(gstream, m[2]);
-  fpfloat(gstream, m[3]);
-  fpfloat(gstream, m[4]);
-  fpfloat(gstream, m[5]);
+  fpfloat(gstream, matrix.m11);
+  fpfloat(gstream, matrix.m12);
+  fpfloat(gstream, matrix.m21);
+  fpfloat(gstream, matrix.m22);
+  fpfloat(gstream, matrix.tX);
+  fpfloat(gstream, matrix.tY);
   fprintf(gstream, "] setmatrix\n");
 }
 
 - (void) GSConcatCTM: (NSAffineTransform *)ctm
 {
-  float m[6];
-  [ctm getMatrix: m];
+  NSAffineTransformStruct matrix = [ctm transformStruct];
+
   fprintf(gstream, "[");
-  fpfloat(gstream, m[0]);
-  fpfloat(gstream, m[1]);
-  fpfloat(gstream, m[2]);
-  fpfloat(gstream, m[3]);
-  fpfloat(gstream, m[4]);
-  fpfloat(gstream, m[5]);
+  fpfloat(gstream, matrix.m11);
+  fpfloat(gstream, matrix.m12);
+  fpfloat(gstream, matrix.m21);
+  fpfloat(gstream, matrix.m22);
+  fpfloat(gstream, matrix.tX);
+  fpfloat(gstream, matrix.tY);
   fprintf(gstream, "] concat\n");
 }
 
@@ -675,9 +675,9 @@ fpfloat(FILE *stream, float f)
 {
   NSBezierPathElement type;
   NSPoint pts[3];
-  int i, count;
+  int i, count = 10;
   float pattern[10];
-  float phase;
+  float phase = 0.0;
 
   [self DPSnewpath];
   [self DPSsetlinewidth: [path lineWidth]];
@@ -717,12 +717,31 @@ fpfloat(FILE *stream, float f)
 
 - (void) GSRectClipList: (const NSRect *)rects: (int)count
 {
-  [self notImplemented: _cmd];
+  int i;
+  NSRect union_rect;
+
+  if (count == 0)
+    return;
+
+  /* 
+     The specification is not clear if the union of the rects 
+     should produce the new clip rect or if the outline of all rects 
+     should be used as clip path.
+  */
+  union_rect = rects[0];
+  for (i = 1; i < count; i++)
+    union_rect = NSUnionRect(union_rect, rects[i]);
+
+  [self DPSrectclip: NSMinX(union_rect) : NSMinY(union_rect)
+	  : NSWidth(union_rect) : NSHeight(union_rect)];
 }
 
 - (void) GSRectFillList: (const NSRect *)rects: (int)count
 {
-  [self notImplemented: _cmd];
+  int i;
+  for (i = 0; i < count; i++)
+    [self DPSrectfill: NSMinX(rects[i]) : NSMinY(rects[i])
+	  : NSWidth(rects[i]) : NSHeight(rects[i])];
 }
 
 /* ----------------------------------------------------------------------- */
