@@ -11,14 +11,15 @@
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
    
-   This library is distributed in the hope that it will be useful,
+   This library is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+   Boston, MA 02111 USA.
    */
 
 #include "w32_Events.h"
@@ -63,17 +64,6 @@
 	[EVENT_WINDOW(hwnd) sendEvent:ev];
     }   
 		  
-#ifdef __WM_MOVE__
-  printf("sending GS_EVENT %d  GS_SUBTYPE %d\n",[ev type],[ev subtype]);
-  printf("HOLD_MENU_FOR_MOVE is %s\n",flags.HOLD_MENU_FOR_MOVE ? "TRUE" : "FALSE");
-  printf("HOLD_MENU_FOR_SIZE is %s\n",flags.HOLD_MENU_FOR_SIZE ? "TRUE" : "FALSE");
-  
-  printf("%s\n",[[self WindowDetail:EVENT_WINDOW(hwnd)] cString]);
-  printf("EVENTLOCATION %s",[[self NSRectDetails:rect] cString]);
-  printf("[hwnd rect] is %s",[[self NSRectDetails:[EVENT_WINDOW(hwnd) frame]] cString]);
-  fflush(stdout);
-#endif
-  
   ev=nil;
   flags.HOLD_MENU_FOR_MOVE=FALSE;
   flags.HOLD_MINI_FOR_MOVE=FALSE;
@@ -99,31 +89,22 @@
     case SIZE_MAXHIDE:
       {
 	// stubbed for future development
-#ifdef __WM_SIZE__
-        printf("got SIZE_MAXHIDE message\n");
-#endif
       }
       break;
     case SIZE_MAXIMIZED:
       {
 	// stubbed for future development
-#ifdef __WM_SIZE__
-	printf("got SIZE_MAXIMIZED message\n");
-#endif
       }
       break;
     case SIZE_MAXSHOW:
       {
 	// stubbed for future development
-#ifdef __WM_SIZE__
-	printf("got SIZE_MAXSHOW message\n");
-#endif
       }
       break;
     case SIZE_MINIMIZED:
       {
       
-	if  (flags.HOLD_MINI_FOR_SIZE==TRUE) //// this is fix for [5,25 bug]
+	if  (flags.HOLD_MINI_FOR_SIZE==TRUE) //// this is fix for [5, 25 bug]
 	  break;
           
 	// make event
@@ -154,8 +135,8 @@
 	      {
 		[EVENT_WINDOW(hwnd) sendEvent:ev];
 		[self resizeBackingStoreFor:hwnd];
-		            if (flags.useWMTaskBar==YES)
-		[EVENT_WINDOW(hwnd) miniaturize:self]; 
+		if ([self usesNativeTaskbar])
+	  	[EVENT_WINDOW(hwnd) miniaturize:self]; 
 	      } 
 	  }  
       }
@@ -191,9 +172,9 @@
 	      {
 		[EVENT_WINDOW(hwnd) sendEvent:ev];
 		[self resizeBackingStoreFor:hwnd];
-		// fixes part one of bug [5,25] see notes
-		         if (flags.useWMTaskBar==YES)
-		[EVENT_WINDOW(hwnd) deminiaturize:self];
+		// fixes part one of bug [5, 25] see notes
+		if ([self usesNativeTaskbar])
+		  [EVENT_WINDOW(hwnd) deminiaturize:self];
 	      } 
 	  } 
       }
@@ -203,17 +184,6 @@
       break;
     }
                       
-#ifdef __WM_SIZE__
-  printf("sending GS_EVENT %d  GS_SUBTYPE %d\n",[ev type],[ev subtype]);
-  printf("[wParam] SIZE_FLAG is %d\n",(int)wParam);
-  printf("HOLD_MENU_FOR_MOVE is %s\n",flags.HOLD_MENU_FOR_MOVE ? "TRUE" : "FALSE");
-  printf("HOLD_MENU_FOR_SIZE is %s\n",flags.HOLD_MENU_FOR_SIZE ? "TRUE" : "FALSE");
-  printf("%s",[[self WindowDetail:EVENT_WINDOW(hwnd)] cString]);
-  printf("size to:%s",[[self NSRectDetails:rect] cString]);
-  printf("[hwnd rect] is %s",[[self NSRectDetails:[EVENT_WINDOW(hwnd) frame]] cString]);
-  fflush(stdout);
-#endif 		  		  
-
   ev=nil;
   flags.HOLD_MENU_FOR_SIZE=FALSE;
   flags.HOLD_MINI_FOR_SIZE=FALSE;
@@ -272,27 +242,21 @@
 
 		            [EVENT_WINDOW(hwnd) sendEvent:ev];
 
-        //printf(" Rect 1 =\n%s",[[self MSRectDetails:drect] cString]);
+        //printf(" Rect 1 =\n%s", [[self MSRectDetails:drect] cString]);
 
    }
 
-   //printf("wParam is %s\n",wParam ? "TRUE" : "FALSE");*/
+   //printf("wParam is %s\n", wParam ? "TRUE" : "FALSE");*/
 }
 
 - (void) decodeWM_WINDOWPOSCHANGEDParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
 {
   // stub for future dev
-#ifdef __TESTEVENT__
-  printf("WM_WINDOWPOSCHANGED\n");
-#endif
 }
 
 - (void) decodeWM_WINDOWPOSCHANGINGParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
 {
   // stub for future dev
-#ifdef __TESTEVENT__
-  printf("WM_WINDOWPOSCHANGING\n");
-#endif
 }
 
 - (LRESULT) decodeWM_GETMINMAXINFOParams: (WPARAM)wParam : (LPARAM)lParam : (HWND)hwnd
@@ -309,11 +273,6 @@
       return 0;
     }
 	  
-#ifdef __GETMINMAXINFO__
-  printf("%s",[[self WindowDetail:EVENT_WINDOW(hwnd)] cString]);
-  printf("%s",[[self MINMAXDetails:mm] cString]);
-  fflush(stdout);
-#endif
   return 0; 
 }
 
@@ -329,13 +288,6 @@
   [self decodeWM_SIZEParams:hwnd :wParam :lParam];
       
   //Make sure DefWindowProc gets called
-	
-#ifdef __EXITSIZEMOVE__
-  NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "EXITSIZEMOVE", hwnd);
-  printf("%s",[[self WindowDetail:EVENT_WINDOW(hwnd)] cString]);
-  fflush(stdout);
-#endif
-	
   return 0;
 }
 
