@@ -19,10 +19,12 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
 */
 
 
+#include <Foundation/NSCharacterSet.h>
 #include <Foundation/NSDictionary.h>
 #include <Foundation/NSString.h>
 #include <Foundation/NSArray.h>
@@ -72,7 +74,7 @@ NSString *win32_font_family(NSString *fontName);
   [super dealloc];
 }
 
-- (float) widthOf: (const char*) s length: (int) len
+- (float) widthOfString: (NSString*)string
 {
   SIZE size;
   HDC hdc;
@@ -80,24 +82,16 @@ NSString *win32_font_family(NSString *fontName);
 
   hdc = GetDC(NULL);
   old = SelectObject(hdc, hFont);
-  GetTextExtentPoint32(hdc, s, len, &size);
+  GetTextExtentPoint32W(hdc,
+    (const unichar*)[string cStringUsingEncoding: NSUnicodeStringEncoding],
+    [string length],
+    &size);
   SelectObject(hdc, old);
   ReleaseDC(NULL, hdc);
 
   return size.cx;
 }
 
-- (float) widthOfString: (NSString*)string
-{
-  const char *s;
-  int len;
-  
-  s = [string cString];
-  len = strlen(s);
-
-  return [self widthOf: s length: len];
-}
-
 - (NSMultibyteGlyphPacking)glyphPacking
 {
   return NSOneByteGlyphPacking;
@@ -105,6 +99,7 @@ NSString *win32_font_family(NSString *fontName);
 
 - (NSSize) advancementForGlyph: (NSGlyph)glyph
 {
+  unichar u = (unichar)glyph;
   HDC hdc;
   float w;
   ABCFLOAT abc;
@@ -112,8 +107,8 @@ NSString *win32_font_family(NSString *fontName);
 
   hdc = GetDC(NULL);
   old = SelectObject(hdc, hFont);
-  //GetCharWidthFloat(hdc, glyph, glyph, &w);
-  GetCharABCWidthsFloat(hdc, glyph, glyph, &abc);
+  // FIXME ... currently a gnustep glyph is a unichar ... what if that changes.
+  GetCharABCWidthsFloatW(hdc, u, u, &abc);
   SelectObject(hdc, old);
   ReleaseDC(NULL, hdc);
 
@@ -124,6 +119,8 @@ NSString *win32_font_family(NSString *fontName);
 
 - (NSRect) boundingRectForGlyph: (NSGlyph)glyph
 {
+  WORD c = (WORD)glyph;
+  WORD windowsGlyph;
   HDC hdc;
   HFONT old;
   GLYPHMETRICS gm;
@@ -131,7 +128,15 @@ NSString *win32_font_family(NSString *fontName);
 
   hdc = GetDC(NULL);
   old = SelectObject(hdc, hFont);
-  if (GDI_ERROR != GetGlyphOutline(hdc, glyph, 
+  // Convert from GNUstep glyph (unichar) to windows glyph.
+  if (GetGlyphIndicesW(hdc, &c, 1, &windowsGlyph, 0) == GDI_ERROR)
+    {
+      SelectObject(hdc, old);
+      ReleaseDC(NULL, hdc);
+NSLog(@"No glyph for U%d", c);
+      return NSMakeRect(0, 0, 0, 0);	// No such glyph
+    }
+  if (GDI_ERROR != GetGlyphOutlineW(hdc, windowsGlyph, 
 				   GGO_METRICS, // || GGO_GLYPH_INDEX
 				   &gm, 0, NULL, NULL))
     {
@@ -167,9 +172,71 @@ NSString *win32_font_family(NSString *fontName);
   return NSMakePoint(0, 0);
 }
 
-- (void) drawString:  (NSString*)string
-	 onDC: (HDC)hdc at: (POINT)p
+- (NSCharacterSet*) coveredCharacterSet
 {
+  if (coveredCharacterSet == nil)
+    {
+      NSMutableCharacterSet	*ms;
+      unsigned	count; 
+      GLYPHSET *gs = 0;
+      HDC hdc;
+      HFONT old;
+
+      hdc = GetDC(NULL);
+      old = SelectObject(hdc, hFont);
+      count = (unsigned)GetFontUnicodeRanges(hdc, 0);
+      if (count > 0)
+	{
+          gs = (GLYPHSET*)objc_malloc(count);
+          if ((unsigned)GetFontUnicodeRanges(hdc, gs) == count)
+	    {
+	      numberOfGlyphs = gs->cGlyphsSupported;
+	      if (gs->flAccel == 1 /* GS_8BIT_INDICES */)
+		{
+		  for (count = 0; count < gs->cRanges; count++)
+		    {
+		      NSRange	range;
+
+		      range.location = gs->ranges[count].wcLow & 0xff;
+		      range.length = gs->ranges[count].cGlyphs;
+		      [ms addCharactersInRange: range];
+		    }
+		}
+	      else
+		{
+		  for (count = 0; count < gs->cRanges; count++)
+		    {
+		      NSRange	range;
+
+		      range.location = gs->ranges[count].wcLow;
+		      range.length = gs->ranges[count].cGlyphs;
+		      [ms addCharactersInRange: range];
+		    }
+		}
+	    }
+          objc_free(gs);
+	}
+      SelectObject(hdc, old);
+      ReleaseDC(NULL, hdc);
+      coveredCharacterSet = [ms copy];
+      RELEASE(ms);
+    }
+  return coveredCharacterSet;
+}
+
+- (void) drawString: (NSString*)string
+	       onDC: (HDC)hdc
+		 at: (POINT)p
+{
+  HFONT old;
+
+  old = SelectObject(hdc, hFont);
+  TextOutW(hdc,
+    p.x,
+    p.y - ascender,
+    (const unichar*)[string cStringUsingEncoding: NSUnicodeStringEncoding],
+    [string length]); 
+  SelectObject(hdc, old);
 }
 
 - (void) draw:(const char*)s length: (int)len 
@@ -187,23 +254,32 @@ NSString *win32_font_family(NSString *fontName);
 	       onDC: (HDC)hdc
 		 at: (POINT)p
 {
+  WORD buf[len];
   HFONT old;
-  WORD	buf[len];
-  int	i;
+  int i;
 
+  old = SelectObject(hdc, hFont);
   /*
    * For now, assume that a glyph is a unicode character and can be
    * stored in a windows WORD
    */
   for (i = 0; i < len; i++)
     {
-      buf[i] = s[i];
+      buf[i] = (WORD)s[i];
     }
-  old = SelectObject(hdc, hFont);
   TextOutW(hdc, p.x, p.y - ascender, buf, len); 
   SelectObject(hdc, old);
 }
 
+- (unsigned) numberOfglyphs
+{
+  if (coveredCharacterSet == nil)
+    {
+      [self coveredCharacterSet];
+    }
+  return numberOfGlyphs;
+}
+
 @end
 
 @implementation WIN32FontInfo (Private)
@@ -211,9 +287,9 @@ NSString *win32_font_family(NSString *fontName);
 - (BOOL) setupAttributes
 {
   HDC hdc;
-  TEXTMETRIC metric;
+  TEXTMETRICW metric;
   HFONT old;
-  LOGFONT logfont;
+  LOGFONTW logfont;
   NSRange range;
 
   //NSLog(@"Creating Font %@ of size %f", fontName, matrix[0]);
@@ -233,8 +309,10 @@ NSString *win32_font_family(NSString *fontName);
     logfont.lfItalic = 1; 
 
   logfont.lfQuality = ANTIALIASED_QUALITY;
-  strncpy(logfont.lfFaceName, [familyName cString], LF_FACESIZE);
-  hFont = CreateFontIndirect(&logfont);
+  wcsncpy(logfont.lfFaceName,
+    (const unichar*)[familyName cStringUsingEncoding: NSUnicodeStringEncoding],
+    LF_FACESIZE);
+  hFont = CreateFontIndirectW(&logfont);
   if (!hFont)
     {
       NSLog(@"Could not create font %@", fontName);
@@ -243,7 +321,7 @@ NSString *win32_font_family(NSString *fontName);
     }
 
   old = SelectObject(hdc, hFont);
-  GetTextMetrics(hdc, &metric);
+  GetTextMetricsW(hdc, &metric);
   SelectObject(hdc, old);
   ReleaseDC(NULL, hdc);
 
@@ -253,6 +331,9 @@ NSString *win32_font_family(NSString *fontName);
   ascender = metric.tmAscent;
   //NSLog(@"Resulted in height %d and ascent %d", metric.tmHeight, metric.tmAscent);
   descender = -metric.tmDescent;
+  /* TODO */
+  xHeight = ascender * 0.5;
+  maximumAdvancement = NSMakeSize((float)metric.tmMaxCharWidth, 0.0);
 
   fontBBox = NSMakeRect((float)(0),
 			(float)(0 - metric.tmAscent),
