@@ -39,7 +39,7 @@ static XWindowBuffer **window_buffers;
 static int num_window_buffers;
 
 
-static int use_shape_hack = 0; /* this is an ugly hack :) */
+static int use_shape_hack = 0; /* this is an ugly hack : ) */
 
 
 static int did_test_xshm = 0;
@@ -47,7 +47,7 @@ static int use_xshm = 1;
 static int num_xshm_test_errors = 0;
 
 static NSString *xshm_warning
-	= @"Falling back to normal XImage:s (will be slower).";
+	= @"Falling back to normal XImage: s (will be slower).";
 
 static int test_xshm_error_handler(Display *d, XErrorEvent *ev)
 {
@@ -95,6 +95,7 @@ static void test_xshm(Display *display, int drawing_depth)
 	DefaultVisual(display, DefaultScreen(display)),
 	drawing_depth, ZPixmap, NULL, &shminfo,
 	1, 1);
+    XSync(display, False);
     if (!ximage || num_xshm_test_errors)
       {
 	NSLog(@"XShm not supported, XShmCreateImage failed.");
@@ -126,7 +127,10 @@ static void test_xshm(Display *display, int drawing_depth)
       }
 
     shminfo.readOnly = 0;
-    if (!XShmAttach(display, &shminfo) || num_xshm_test_errors)
+    if (!XShmAttach(display, &shminfo))
+       num_xshm_test_errors++;
+    XSync(display, False);
+    if (num_xshm_test_errors)
       {
 	NSLog(@"XShm not supported, XShmAttach() failed.");
 	XDestroyImage(ximage);
@@ -149,7 +153,7 @@ static void test_xshm(Display *display, int drawing_depth)
     if (num_xshm_test_errors)
       {
 	NSLog(@"XShm not supported.");
-no_xshm:
+no_xshm: 
 	NSLog(xshm_warning);
 	use_xshm = 0;
       }
@@ -268,7 +272,7 @@ no_xshm:
       if (!use_xshm)
         goto no_xshm;
 
-      /* Use XShm if possible, else fall back to normal XImage:s */
+      /* Use XShm if possible, else fall back to normal XImage: s */
       wi->use_shm = 1;
       wi->ximage = XShmCreateImage(wi->display,
 	DefaultVisual(wi->display, DefaultScreen(wi->display)),
@@ -343,7 +347,7 @@ no_xshm:
 
       if (!wi->ximage)
 	{
-no_xshm:
+no_xshm: 
 	  wi->use_shm = 0;
 	  wi->ximage = XCreateImage(wi->display, DefaultVisual(wi->display,
 	    DefaultScreen(wi->display)), aDI->drawing_depth, ZPixmap, 0, NULL,
@@ -384,7 +388,7 @@ no_xshm:
 
 extern int XShmGetEventBase(Display *d);
 
--(void) _gotShmCompletion
+- (void) _gotShmCompletion
 {
   if (!use_shm)
     return;
@@ -421,7 +425,7 @@ extern int XShmGetEventBase(Display *d);
 //	XFlush(window->display);
 }
 
--(void) _exposeRect: (NSRect)rect
+- (void) _exposeRect: (NSRect)rect
 {
 /* TODO: Somehow, we can get negative coordinates in the rectangle. So far
 I've tracked them back to [NSWindow flushWindow]. Should probably figure
@@ -435,7 +439,7 @@ rects in the new size before we are updated.
 For now, we just intersect with our known size to avoid problems with X.
 
 And, to avoid problems with clever optimizations and float vs. double
-accuracy, we do the test using int:s.
+accuracy, we do the test using int: s.
 */
   int x, y, w, h;
 
@@ -454,13 +458,13 @@ accuracy, we do the test using int:s.
       h += y;
       y = 0;
     }
-  if (x + w > window->xframe.size.width)
+  if (x + w > sx)
     {
-      w = window->xframe.size.width - x;
+      w = sx - x;
     }
-  if (y + h > window->xframe.size.height)
+  if (y + h > sy)
     {
-      h = window->xframe.size.height - y;
+      h = sy - y;
     }
 
   if (w <= 0 || h <= 0)
@@ -596,20 +600,21 @@ static int warn = 0;
 	 events. */
       {
 	XEvent e;
+
 	while (XCheckTypedEvent(window->display,
-	    XShmGetEventBase(window->display) + ShmCompletion,
-	    &e))
+	  XShmGetEventBase(window->display) + ShmCompletion, &e))
 	  {
 	    [isa _gotShmCompletion: ((XShmCompletionEvent *)&e)->drawable];
 	  }
       }
     }
   else if (ximage)
-    XPutImage(display, drawable, gc, ximage,
-	      x, y, x, y, w, h);
+    {
+      XPutImage(display, drawable, gc, ximage, x, y, x, y, w, h);
+    }
 }
 
--(void) needsAlpha
+- (void) needsAlpha
 {
   if (has_alpha)
     return;
@@ -645,7 +650,7 @@ static int warn = 0;
   memset(alpha, 0xff, sx * sy);
 }
 
--(void) dealloc
+- (void) dealloc
 {
   int i;
 
@@ -681,14 +686,14 @@ static int warn = 0;
 }
 
 
-+(void) initialize
++ (void) initialize
 {
   NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
   use_shape_hack = [ud boolForKey: @"XWindowBuffer-shape-hack"];
 }
 
 
-+(void) _gotShmCompletion: (Drawable)d
++ (void) _gotShmCompletion: (Drawable)d
 {
   int i;
   for (i = 0; i < num_window_buffers; i++)
