@@ -43,6 +43,7 @@
 #include <Foundation/NSData.h>
 #include <Foundation/NSValue.h>
 // For the encoding functions
+#include <GNUstepBase/GSMime.h>
 #include <GNUstepBase/Unicode.h>
 
 static Atom XA_SLANT = (Atom)0;
@@ -306,8 +307,7 @@ static BOOL XGInitAtoms(Display *dpy)
     (float)(0 - font_info->max_bounds.ascent),
     (float)(font_info->max_bounds.rbearing - font_info->min_bounds.lbearing),
     (float)(font_info->max_bounds.ascent + font_info->max_bounds.descent));
-  maximumAdvancement = NSMakeSize(font_info->max_bounds.width,
-    (font_info->max_bounds.ascent +font_info->max_bounds.descent));
+  maximumAdvancement = NSMakeSize(font_info->max_bounds.width, 0.0);
   minimumAdvancement = NSMakeSize(0,0);
   weight = XGWeightOfFont(xdpy, font_info);
   traits = XGTraitsOfFont(xdpy, font_info);
@@ -319,13 +319,22 @@ static BOOL XGInitAtoms(Display *dpy)
 
       if (enc != nil)
         {
-	  mostCompatibleStringEncoding = GSEncodingForRegistry(reg, enc);
-	  if (mostCompatibleStringEncoding == GSUndefinedEncoding)
-	    mostCompatibleStringEncoding = NSASCIIStringEncoding;
-	  encodingScheme = [NSString stringWithFormat: @"%@-%@", 
-				     reg, enc];
+	  if ([enc length] != 0 && [enc isEqualToString: @"0"] == NO)
+	    {
+	      encodingScheme = [NSString stringWithFormat: @"%@-%@", reg, enc];
+	    }
+	  else
+	    {
+	      encodingScheme = reg;
+	    }
+	  mostCompatibleStringEncoding
+	    = [GSMimeDocument encodingFromCharset: encodingScheme];
 	  NSDebugLog(@"Found encoding %d for %@", 
 		     mostCompatibleStringEncoding, encodingScheme);
+	  if (mostCompatibleStringEncoding == GSUndefinedEncoding)
+	    {
+	      mostCompatibleStringEncoding = NSASCIIStringEncoding;
+	    }
 	  RETAIN(encodingScheme);
 	}
     }
