@@ -19,7 +19,8 @@
    
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111 USA.
+   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02111 USA.
    */
 
 #include "config.h"
@@ -244,6 +245,8 @@ static	Region	emptyRegion;
     }
 
   XSetRegion(XDPY, xgcntxt, clipregion);
+NSDebugLLog(@"XGGraphics", @"Clip %@ set to X rect %@",
+  self, NSStringFromRect([self clipRect]));
 }
 
 /* Returns the clip region, which must be freed by the caller */
@@ -417,6 +420,8 @@ static	Region	emptyRegion;
 - (void) copyBits: (XGGState*)source fromRect: (NSRect)aRect 
 				      toPoint: (NSPoint)aPoint
 {
+  NSAffineTransformStruct	sctms;
+  NSAffineTransformStruct	ctms;
   XRectangle	dst;
   XRectangle    src;
   NSRect	flushRect;
@@ -439,12 +444,14 @@ static	Region	emptyRegion;
   flushRect.size = aRect.size;
   flushRect.origin = aPoint;
   dst = XGViewRectToX(self, flushRect);
-  if (source->ctm->matrix.m22 < 0 && ctm->matrix.m22 > 0) dst.y += src.height;
-  if (source->ctm->matrix.m22 > 0 && ctm->matrix.m22 < 0) dst.y -= src.height;
+  sctms = [source->ctm transformStruct];
+  ctms = [ctm transformStruct];
+  if (sctms.m22 < 0 && ctms.m22 > 0) dst.y += src.height;
+  if (sctms.m22 > 0 && ctms.m22 < 0) dst.y -= src.height;
   NSDebugLLog(@"XGGraphics", @"Copy area from %@ to %@",
-	      NSStringFromRect(aRect), NSStringFromPoint(aPoint));
+    NSStringFromRect(aRect), NSStringFromPoint(aPoint));
   XCopyArea(XDPY, from, draw, xgcntxt,
-                src.x, src.y, src.width, src.height, dst.x, dst.y);
+    src.x, src.y, src.width, src.height, dst.x, dst.y);
 }
 
 - (void) _alphaBuffer: (gswindow_device_t *)dest_win
@@ -1283,7 +1290,7 @@ static	Region	emptyRegion;
   /* Note we update the current point according to the current 
      transformation scaling, although the text isn't currently
      scaled (FIXME). */
-  scale = [ctm sizeInMatrixSpace: NSMakeSize(1, 1)];
+  scale = [ctm transformSize: NSMakeSize(1, 1)];
   //scale = NSMakeSize(1, 1);
   [path relativeMoveToPoint: NSMakePoint(width * scale.width, 0)];
 }
@@ -1330,7 +1337,7 @@ static	Region	emptyRegion;
   /* Note we update the current point according to the current 
      transformation scaling, although the text isn't currently
      scaled (FIXME). */
-  scale = [ctm sizeInMatrixSpace: NSMakeSize(1, 1)];
+  scale = [ctm transformSize: NSMakeSize(1, 1)];
   //scale = NSMakeSize(1, 1);
   [path relativeMoveToPoint: NSMakePoint(width * scale.width, 0)];
 }
@@ -1420,7 +1427,7 @@ static	Region	emptyRegion;
   int	w;
   NSSize	ws;
 
-  ws = [ctm sizeInMatrixSpace: NSMakeSize(width,width)];
+  ws = [ctm transformSize: NSMakeSize(width,width)];
   width = (ws.width + ws.height) / 2;
 
   /*
@@ -1484,10 +1491,12 @@ static	Region	emptyRegion;
 - (void)DPSrectclip: (float)x : (float)y : (float)w : (float)h 
 {
   XRectangle    xrect;
+  NSRect	orect;
 
   CHECK_GC;
 
-  xrect = XGViewRectToX(self, NSMakeRect(x, y, w, h));
+  orect = NSMakeRect(x, y, w, h);
+  xrect = XGViewRectToX(self, orect);
 
   if (clipregion == 0)
     {
@@ -1521,7 +1530,10 @@ static	Region	emptyRegion;
     [self setColor: &fillColor state: COLOR_FILL];
 
   bounds = XGViewRectToX(self, NSMakeRect(x, y, w, h));
-  // Hack: Only draw when alpha is not zero
+NSDebugLLog(@"XGGraphics", @"Fill %@ X rect %d,%d,%d,%d",
+  self, bounds.x, bounds.y, bounds.width, bounds.height);
+
+// Hack: Only draw when alpha is not zero
   if (drawingAlpha == NO || fillColor.field[AINDEX] != 0.0)
     XFillRectangle(XDPY, draw, xgcntxt,
 		   bounds.x, bounds.y, bounds.width, bounds.height);
@@ -1794,7 +1806,7 @@ static	Region	emptyRegion;
 
   // --- determine region to read --------------------------------------
 
-  rect.origin = [ctm pointInMatrixSpace: rect.origin];
+  rect.origin = [ctm transformPoint: rect.origin];
   srect = XGWindowRectToX(self, rect);
   srect = XGIntersectionRect (srect, accessibleRectForWindow (source_win));
   ssize.width = srect.width;
